/*
 * 环境变量,
 * 目录结构
 * 等配置文件
 */

const path = require('path');
const fs = require('fs');

const util = require('../util/index.js');

let options = {
  babel: false,
  vue: false,
  react: false,
  ssl: true,
  watch: false,
  development_dist: 'src/web_inf/' + process.env.LUNA_APP_SYSTEM + '/dev',
  production_dist: 'src/web_inf/' + process.env.LUNA_APP_SYSTEM + '/online',
  exclude: ['common', 'vue-common'], // 忽略的目录，该目录不会作为入口
  externals: {}, // 官方externals
  ignoreJs: [], // 忽略打包的js
  ignoreCss: [], // 忽略打包的css
  cache: true, // 是否开启缓存
  htmlMinify: true, // 是否启用html压缩
  htmlAttrIgnore: [], // 忽略验证的 html 属性
};

const lunarc = path.isAbsolute(process.env.CONFIG_FILE) ?
  process.env.CONFIG_FILE :
  path.resolve(process.env.LUNA_WORKING_PATH, '.lunarc');

if (fs.existsSync(lunarc)) {
  options.lunarc = lunarc;
  options = Object.assign(
    options,
    JSON.parse(fs.readFileSync(lunarc, 'utf8')),
  );
}

options.ignoreFile = options.ignoreJs.concat(options.ignoreCss);
options.ignoreFileRegExp = new RegExp(options.ignoreFile.map(item => '(' + item.replace('.', '\\.') + ')').join('|') + '$', 'i');

// 将环境变量统一写入配置中
options = Object.assign(
  options,
  {
    NODE_ENV: process.env.NODE_ENV,
    page: process.env.LUNA_PAGE,
    pages: process.env.LUNA_PAGE === 'all' ? util.readDir(path.resolve(process.env.LUNA_WORKING_PATH, 'src'), options.exclude) : [process.env.LUNA_PAGE],
    action: process.env.LUNA_ACTION,
    luna_root: process.env.LUNA_ROOT,
    working_path: process.env.LUNA_WORKING_PATH,
    project_root: process.env.LUNA_PROJECT_ROOT,
    project_name: process.env.LUNA_PROJECT_NAME,
    app_system: process.env.LUNA_APP_SYSTEM,
    debug: process.env.DEBUG === 'true',
    debug_port: process.env.DEBUG_PORT,
    user: process.env.USER,
    cachePath: path.resolve(process.env.LUNA_WORKING_PATH, '.luna-cache/'),
  },
);

module.exports = options;
