const path = require('path');
/*
 * npm install extract-text-webpack-plugin@next  如果安装3.x的话，编译不过去
 */
const ExtractTextPlugin = require('extract-text-webpack-plugin');

const LessPluginFunctions = require('less-plugin-functions');

const config = require('../index.js');
const improve = require('./improve.js');

const postCssLoader = {
  loader: 'postcss-loader',
  options : {
    ident: 'postcss',
    plugins: (loader) => [
      require('autoprefixer')({ browsers: ['last 5 versions'] })
    ]
  }
};

const loaders = {
  art: {
    test: /\.art$/i,
    exclude: improve.baseExclude,
    use: [{
      loader: 'art-template-loader',
      options: {
        debug: config.NODE_ENV !== 'production',
        compileDebug: config.NODE_ENV !== 'production',
      },
    }, 'extract-loader', {
      loader: 'html-loader',
      options: {
        attrs: ['img:src'],
      },
    }],
  },
  hbs: {
    test: /\.hbs$/i,
    exclude: improve.baseExclude,
    use: [{
      loader: 'handlebars-loader',
      options: {
        helperDirs: [path.resolve(config.working_path, 'src/common/helpers/')],
      },
    }, 'extract-loader', {
      loader: 'html-loader',
      options: {
        attrs: ['img:src'],
      },
    }],
  },
  html: {
    test: /\.html$/i,
    exclude: improve.baseExclude,
    // use: 'html-loader'
    /* 这里使用underscore-templte的原因是html-loader无法抽离link和srcipt的外链属性，从而无法进行CDN替换
    ** 如果不适用cdn或者不适用underscore模板的话，直接使用html-loader就好。
    */
    use: [{
      loader: 'underscore-template-loader',
      options: {
        attributes: ['img:src', 'link:href', 'script:src'],
        interpolate: '\\{\\[(.+?)\\]\\}',
        evaluate: '\\{%([\\s\\S]+?)%\\}',
        escape: '\\[\\[(.+?)\\]\\]', //
      },
    }],
  },
  css: {
    test: /\.css$/i,
    use: ExtractTextPlugin.extract({
      fallback: 'style-loader',
      use: [{
        loader: 'css-loader',
        options: {
          minimize: true,
        },
      }],
    }),
  },
  less: {
    test: /\.less$/i,
    exclude: improve.baseExclude,
    use: ExtractTextPlugin.extract({
      fallback: 'style-loader',
      use: [{
        loader: 'css-loader',
        options: {
          minimize: true,
        },
      },
      postCssLoader,
      {
        loader: 'less-loader',
        options: {
          javascriptEnabled: true,
          plugins: [new LessPluginFunctions()],
        },
      }],
    }),
  },
  image: {
    test: /\.(jpe?g|png|gif|svg)$/i,
    exclude: improve.baseExclude,
    use: [{
      loader: 'url-loader',
      options: {
        limit: 10,
        name: 'img/[hash:16].[ext]',
      },
    }],
  },
  ignoreFile: {
    test: config.ignoreFileRegExp,
    use: [{
      loader: 'url-loader',
      options: {
        limit: 1,
        name: '[ext]/[name].[hash:16].[ext]',
      },
    }],
  },
  js: {
    test: /\.js$/,
    exclude: [improve.baseExclude],
    use: ['babel-loader'],
  },
  font: {
    test: /\.(eot|woff|woff2|ttf|otf|svg)$/i,
    use: ['file-loader'],
  },
  vue: {
    test: /\.vue$/,
    use: 'vue-loader'
  }
};

// 根据环境使用定制loader
// 目前V1版本不支持上传CDN
if (config.NODE_ENV === 'production') {
  /*
  const cdnLoader = {
    loader: 'muse-cdn-loader',
    options: {
      ssl: config.ssl,
      iSSLHost: config.iSSLHost,
      sSSLHost: config.sSSLHost,
      cache: path.resolve(config.cachePath, 'muse-cdn-loader.json'),
    },
  };
  loaders.image.use = [cdnLoader];
  loaders.ignoreFile.use = [cdnLoader];
  loaders.font.use = [cdnLoader];
  */
}

/* 暂时屏蔽
if (config.NODE_ENV === 'development') {
  // 开发环境使用 style-loader ，支持 HMR
  loaders.less = {
    test: /\.less$/i,
    exclude: improve.baseExclude,
    use: [
      'style-loader',
      'css-loader',
      postCssLoader,
      {
        loader: 'less-loader',
        options: {
          javascriptEnabled: true,
          plugins: [new LessPluginFunctions()],
        },
      },
    ],
  };
}
*/

const rules = [
  loaders.art,
  loaders.hbs,
  loaders.html,
  loaders.css,
  loaders.less,
  loaders.image,
  loaders.font,
];

// 如果有忽略文件，测处理下对应文件的exclude，以及添加忽略文件的loader
if (config.ignoreFile.length > 0) {
  loaders.css.exclude = [config.ignoreFileRegExp];
  loaders.js.exclude.push(config.ignoreFileRegExp);
  rules.push(loaders.ignoreFile);
}

// 根据条件准备loader
if (config.babel || config.vue || config.react) {
  rules.push(loaders.js);
}

if (config.vue) {
  rules.push(loaders.vue);
}

module.exports = {
  rules,
};
