const path = require('path');
const webpack = require('webpack');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const { VueLoaderPlugin } = require('vue-loader');
const { BundleAnalyzerPlugin } = require('webpack-bundle-analyzer');
const HardSourceWebpackPlugin = require('hard-source-webpack-plugin');

const config = require('../index.js');

const plugins = [
  new webpack.ContextReplacementPlugin(/moment[/\\]locale$/, /zh-cn/),
  new ExtractTextPlugin({
    filename: 'css/[name].[chunkhash:16].css',
    allChunks: true,
  }),
  new HardSourceWebpackPlugin({
    cacheDirectory: path.resolve(config.cachePath, 'hard-source/'),
    environmentHash: {
      root: config.working_path,
    },
  }),
  new HardSourceWebpackPlugin.ExcludeModulePlugin([{
    test: /handlebars-loader/,
  }])
];


if (config.vue) {
  plugins.push(new VueLoaderPlugin());
}

const htmlWebpackPluginMinify = (config.NODE_ENV !== 'production' || config.htmlMinify === false) ? false : {
  removeComments: true,
  collapseWhitespace: true,
  minifyJS: true,
  minifyCSS: true,
};

config.pages.forEach((page) => {
  plugins.push(new HtmlWebpackPlugin({
    filename: page + '.html',
    template: path.resolve(config.working_path, 'src', page, 'static', page + '.html'),
    minify: htmlWebpackPluginMinify,
    chunks: [page],
    inject: true,
  }));
});

if (config.debug) {
  plugins.push(new BundleAnalyzerPlugin({
    analyzerHost: '0.0.0.0',
    analyzerPort: config.debug_port,
    defaultSizes: 'parsed',
  }));
}

module.exports = plugins;
