// bundle启动脚本

const path = require('path');
const fs = require('fs');
const fse = require('fs-extra');
const { spawn } = require('child_process');
const colors = require('colors');
const readlineSync = require('readline-sync');

const util = require('./util/index.js');

const lunaRoot= path.resolve(__dirname);
const runPath = process.cwd();

module.exports = (options) => {
  util.print('Using luna', c => c.magenta(lunaRoot));
  util.print('Current directory', c => c.magenta(runPath));
  util.print(c => c.green('Start detection working path...'));

  let workingPath;
  let projectRoot;
  let appSystem = 'front';
  const runPathArr = runPath.split(path.sep);
  // console.log(runPathArr);

  // TODO 根据项目模板有可能会更改,更好的是传变量，目前先写死
  if (fs.existsSync(path.resolve(runPath, 'websrc'))) {
    projectRoot = runPath;
  } else if (runPathArr[runPathArr.length - 1] === 'websrc') {
    projectRoot = path.resolve('..');
  } else if (runPathArr[runPathArr.length - 2] === 'websrc') {
    projectRoot = path.resolve('..', '..');
  }

  if (!projectRoot) {
    util.print(c => c.red('Can`t found working path. Please make sure that you are in the correct directory!'));
    return;
  }

  const projectRootArr = projectRoot.split(path.sep);
  const systemDir = util.readDir(path.resolve(projectRoot, 'websrc'));

  if (systemDir.length === 0) {
    util.print(c => c.red('Can`t found APP_SYS. You should build project under `' + path.resolve(projectRoot, 'websrc') + '`'));
    return;
  }
  if (systemDir.length === 1) {
    appSystem = systemDir[0];
    workingPath = path.resolve(projectRoot, 'websrc', systemDir[0]);
  } else if (systemDir.indexOf(runPathArr[runPathArr.length - 1]) > -1) { // 这个case是多个front或多个admin场景，几乎用不到
    appSystem = runPathArr[runPathArr.length - 1];
    workingPath = path.resolve(projectRoot, 'websrc', runPathArr[runPathArr.length - 1]);
  } else {
    const output = ['There are some front systems as below: '];
    systemDir.forEach((v, i) => {
      output.push(c => c.green('\n[' + (i + 1) + '] ' + v));
    });
    util.print.apply(util.print, output);
    const userChoose = readlineSync.question('Please choose one (default:1): ');
    appSystem = systemDir[userChoose - 1] ? systemDir[userChoose - 1] : systemDir[0];
    workingPath = path.resolve(projectRoot, 'websrc', appSystem);
  }

  /*
   * 有效的命令有 build,watch, deploy, bdonline.
   * deploy为静态资源上传CDN，TODO v1.x暂不实现
   * bdonline = build online, 让webpack编译时使用production的配置，但是静态资源不上cdn的命令。
   * 初始化process.env.NODE_ENV非常重要，决定了后续webpack.config.js等流程
   */
  process.env.NODE_ENV = ['deploy', 'bdonline'].indexOf(options.action) > -1 ? 'production' : 'development';
  process.env.DEBUG = options.debug;
  process.env.DEBUG_PORT = options.debugPort || 58120;
  process.env.LUNA_ROOT = lunaRoot;
  process.env.LUNA_ACTION = options.action;
  process.env.CONFIG_FILE = options.config_file;
  process.env.LUNA_PAGE = options.page;
  process.env.LUNA_WORKING_PATH = workingPath;
  process.env.LUNA_PROJECT_ROOT = projectRoot;
  process.env.LUNA_PROJECT_NAME = projectRootArr[projectRootArr.length - 1];
  process.env.LUNA_APP_SYSTEM = appSystem;
  process.env.LUNA_CMD_RUN_PATH = runPath;

  /*
   * luna配置初始化完毕，开始进行bundle控制流程
   * **deploy 不允许对单个页面进行发布
   */

  if ((options.action === 'deploy' || options.action === 'bdonline') && process.env.LUNA_PAGE !== 'all') {
    util.print(c => c.red('Deploy single page is forbidden!'));
    util.print(c => c.red('I hope you know what are you doing now!'));
    const nextStep = readlineSync.question(`Continue to deploy all pages ? (yes/no):`);
    if (nextStep === 'yes') process.env.LUNA_PAGE = 'all';
  }

  // 引入webpack config
  const config = require('./config');

  if (config.action === 'watch' && config.watch !== true) {
    util.print(c => c.red('Couldn`t use watch command. Please make sure your project support watch mode'));
    return;
  }
  if (config.action === 'build' && config.watch === true) {
    util.print(c => c.red('You`re in watch mode. Can`t use build command'));
    return;
  }
  if (config.lunarc) {
    util.print('Use Config', c => c.magenta(config.lunarc));
  }

  util.print('Starting task', c => c.magenta(`${config.action} ${config.page}`));

  const webpackBin = path.resolve(__dirname, 'node_modules', '.bin/webpack');

  const webpackConfig = path.resolve(__dirname, 'config', 'webpack.' + config.action + '.js');

  if (config.action === 'build') {
    spawn(webpackBin, ['--config', webpackConfig], {
      stdio: 'inherit',
    });
  } else if (config.action === 'watch') {
    spawn(webpackBin, ['--config', webpackConfig, '--watch'], {
      stdio: 'inherit',
    });
  } else if (config.action === 'bdonline') {
    spawn(webpackBin, ['--config', webpackConfig], {
      stdio: 'inherit',
    });
  } else {
    // TODO right now, deploy is the same as bdonline
    spawn(webpackBin, ['--config', webpackConfig], {
      stdio: 'inherit',
    });
  }
};
