'use strict';

const fs = require('fs');
const path = require('path');
const moment = require('moment');
const colors = require('colors');

// 读取目录下的所有文件夹，忽略隐藏文件夹，以及自定义忽略
const readDir = (dirname, exclude = []) => {
  const dirs = fs.readdirSync(dirname);
  return dirs.filter((item) => {
    const stats = fs.statSync(path.resolve(dirname, item));
    if (!stats.isDirectory()) return false;
    if (item.indexOf('.') === 0) return false;
    if (exclude.indexOf(item) >= 0) return false;
    return true;
  });
};

// 读取目录下的所有文件，忽略隐藏文件
const readFiles = (dirname, baseDir = '') => {
  let ret = [];
  const dirs = fs.readdirSync(dirname);
  dirs.forEach((item) => {
    const itemPath = path.resolve(dirname, item);
    const stats = fs.statSync(itemPath);
    if (stats.isFile()) ret.push(path.join(baseDir, item));
    if (stats.isDirectory()) {
      ret = ret.concat(readFiles(itemPath, path.join(baseDir, item)));
    }
  });
  return ret;
};

// 输出到命令行
const print = (...args) => {
  args.unshift('[' + colors.gray(moment().format('HH:mm:ss')) + ']');
  args.forEach((item, index) => {
    if (typeof item === 'function') {
      args[index] = item(colors);
    }
  });
  console.log(...args);
};

module.exports = {
  readDir,
  readFiles,
  print,
};
